package coin

import (
	"context"
	"github.com/pkg/errors"
	"live-service/common/nerr"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetUserCoinLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetUserCoinLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetUserCoinLogic {
	return &GetUserCoinLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// getUserCoin 获取用户弹币
func (l *GetUserCoinLogic) GetUserCoin(in *pb.UserIdReq) (*pb.GetUserCoinResp, error) {
	// 查询当前用户积分
	coin, err := l.svcCtx.CoinManager.GetUserCoin(l.ctx, in.UserId)
	if err != nil {
		return nil, errors.Wrapf(nerr.NewWithCode(nerr.DBError), "查询用户弹币失败, err:%+v", err)
	}

	return &pb.GetUserCoinResp{
		UserId:  in.UserId,
		Current: coin,
	}, nil
}
