package gift

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/uuid"
	"github.com/pkg/errors"
	pbVars "live-service/app/pb/vars"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/common/coin_manager"
	"live-service/app/user_center/rpc/internal/logic/gift_collect"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	"live-service/common/nerr"
	"strconv"

	"github.com/zeromicro/go-zero/core/logx"
)

type UserSendGiftLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewUserSendGiftLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UserSendGiftLogic {
	return &UserSendGiftLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// UserSendGift 用户赠送礼物
// 1. 记录用户赠送信息
// 2. 新增用户积分
func (l *UserSendGiftLogic) UserSendGift(in *pb.UserSendGiftReq) (*pb.Empty, error) {
	// 记录送礼信息
	{
		dbUserGift := &model.UserGift{
			Id:       uuid.NextId(),
			UserId:   in.UserId,
			Platform: in.Platform,
			RoomId:   in.RoomId,
			GiftId:   strconv.FormatInt(in.GiftId, 10),
			GiftName: in.GiftName,
			Num:      in.Num,
		}
		if in.IsPaid {
			dbUserGift.Price = in.Price
		} else {
			dbUserGift.FreePrice = in.Price
		}
		if err := l.svcCtx.UserGiftModel.Insert(l.ctx, nil, dbUserGift); err != nil {
			l.Logger.Errorf("记录用户[%d]送礼信息 [%d:%s:%d] 失败，操作继续...", in.UserId, in.GiftId, in.GiftName, in.Num)
		}
	}

	// 弹币
	{
		addonCoin := calcCoin(l.svcCtx.GameConfig, in.Platform, in.IsPaid, float64(in.Price), in.Num)
		if tmpMap, ok := gift_collect.Service.GetCacheByPlatform(in.Platform); ok {
			if tmpData, ok := tmpMap[in.GiftId]; ok {
				addonCoin = calcCoin(l.svcCtx.GameConfig, in.Platform, tmpData.IsPaid, tmpData.Price, in.Num)
			}
		}
		_, err := l.svcCtx.CoinManager.ChangeCoin(l.ctx, nil, &coin_manager.ChangeCoinReq{
			BattleId: in.BattleId,
			UserId:   in.UserId,
			Change:   addonCoin,
		}, pbVars.UserCoinChangedReason_SendGift)
		if err != nil {
			return nil, errors.Wrapf(nerr.NewWithErr(err), "添加弹币失败 err:%v", err)
		}
	}
	return &pb.Empty{}, nil
}
