package statistics

import (
	"context"
	"github.com/pkg/errors"
	pbVars "live-service/app/pb/vars"
	"live-service/app/user_center/model"
	"live-service/common/nerr"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type UserRankPvpLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewUserRankPvpLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UserRankPvpLogic {
	return &UserRankPvpLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *UserRankPvpLogic) UserRankPvp(in *pb.UserRankReq) (*pb.UserRankResp, error) {
	// 1. 查询用户自己当前分数
	// 2. 获取排名
	// 3. 所有排名信息（需要首先判断）
	var dbScoreMap map[string]int64
	var err error
	if in.AllRankType {
		//in.UserId
		if dbScoreMap, err = l.svcCtx.StatisticsPvpModel.FindScoreByType(l.ctx, nil, in.UserId, model.AllScoreType()); err != nil {
			return nil, errors.Wrapf(nerr.NewError(nerr.RankStatisticsSelectError, "获取用户分数失败"), "获取用户分数失败 err:%v", err)
		}
	} else {
		// 只查询一种类型
		scoreTypes := []model.ScoreType{model.ScoreTypeByRankType(in.RankType)}
		if dbScoreMap, err = l.svcCtx.StatisticsPvpModel.FindScoreByType(l.ctx, nil, in.UserId, scoreTypes); err != nil {
			return nil, errors.Wrapf(nerr.NewError(nerr.RankStatisticsSelectError, "获取用户分数失败"), "获取用户分数失败 err:%v", err)
		}
	}
	items := make([]*pb.UserRankResp_Item, 0, len(dbScoreMap))
	for scoreType, score := range dbScoreMap {
		items = append(items, l.rankItem(model.RankTypeByScoreType(model.ScoreType(scoreType)), score))
	}

	return &pb.UserRankResp{
		Items: items,
	}, nil
}

func (l *UserRankPvpLogic) rankItem(rankType pbVars.RankType, score int64) *pb.UserRankResp_Item {
	return &pb.UserRankResp_Item{
		RankType: rankType,
		Score:    score,
		Pos:      Service.RankByScore(rankType, int32(score)),
	}
}
