package user

import (
	"context"
	"github.com/pkg/errors"
	"live-service/common/nerr"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetUserGradeLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetUserGradeLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetUserGradeLogic {
	return &GetUserGradeLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

//  @ZeroGroup: grade
func (l *GetUserGradeLogic) GetUserGrade(in *pb.UserIdReq) (*pb.UserGradeResp, error) {
	gradeResp, err := l.svcCtx.GradeManager.QueryUser(l.ctx, in.UserId)
	if err != nil {
		return nil, errors.Wrapf(nerr.NewWithErr(err), "获取用户段位信息失败, err:%v", err)
	}

	return &pb.UserGradeResp{
		UserId: in.UserId,
		Grade: &pb.Grade{
			Grade:      int32(gradeResp.Grade),
			Level:      int32(gradeResp.Level),
			Star:       int32(gradeResp.Star),
			BravePoint: gradeResp.BravePoint,
		},
	}, nil
}
