package zhg

import (
	"context"
	"github.com/pkg/errors"
	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
	pbVars "live-service/app/pb/vars"
	"live-service/app/user_center/rpc/internal/common/coin_manager"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	"live-service/common/nerr"
	"time"
)

type BuyEliteLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewBuyEliteLogic(ctx context.Context, svcCtx *svc.ServiceContext) *BuyEliteLogic {
	return &BuyEliteLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *BuyEliteLogic) BuyElite(in *pb.EliteReq) (*pb.BuyEliteResp, error) {
	elite, err := l.svcCtx.GameConfig.Elite.FindEliteCfgByLive(in.Sort)
	if err != nil {
		return nil, nerr.NewError(nerr.EliteConfigNotFoundErr, err.Error())
	}
	if elite.Id == 0 {
		return nil, nerr.NewError(nerr.UserBuyDefaultEliteErr, err.Error())
	}

	resp := &pb.BuyEliteResp{
		UserId:   in.UserId,
		EliteId:  elite.Id,
		Cost:     elite.PriceDay,
		Duration: 1,
	}
	if err = l.svcCtx.Db.Transaction(func(tx *gorm.DB) error {
		// 扣减弹币
		if _, err = l.svcCtx.CoinManager.ChangeCoin(l.ctx, tx, &coin_manager.ChangeCoinReq{
			UserId: in.UserId,
			Change: -elite.PriceDay,
		}, pbVars.UserCoinChangedReason_BuyEliteUnit); err != nil {
			return err
		}

		if err = l.svcCtx.UserEliteModel.Addon(l.ctx, nil, in.UserId, elite.Id, 24*time.Hour, false); err != nil {
			return errors.Wrapf(nerr.NewWithErr(err), "兑换精英单位失败 err:%v", err)
		}

		return nil
	}); err != nil {
		return nil, err
	}
	return resp, nil
}
