package zhg

import (
	"context"
	"github.com/pkg/errors"
	"gorm.io/gorm"
	pbVars "live-service/app/pb/vars"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/common/coin_manager"
	"live-service/common/nerr"
	"time"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type BuyTitleLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewBuyTitleLogic(ctx context.Context, svcCtx *svc.ServiceContext) *BuyTitleLogic {
	return &BuyTitleLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *BuyTitleLogic) BuyTitle(in *pb.TitleReq) (*pb.BuyTitleResp, error) {
	titleCfg, err := l.svcCtx.GameConfig.Title.FindTitleCfgByLive(in.Sort)
	if err != nil {
		return nil, nerr.NewError(nerr.TitleConfigNotFoundErr, err.Error())
	}
	if titleCfg.Type != model.TitleTypeDefault {
		return nil, nerr.NewError(nerr.TitleCannotBuyErr, "该称号禁止兑换")
	}

	resp := &pb.BuyTitleResp{
		UserId:   in.UserId,
		TitleId:  titleCfg.Id,
		Name:     titleCfg.Name,
		Cost:     titleCfg.PriceDay,
		Duration: 1,
	}
	if err = l.svcCtx.Db.Transaction(func(tx *gorm.DB) error {
		// 扣减弹币
		if _, err = l.svcCtx.CoinManager.ChangeCoin(l.ctx, tx, &coin_manager.ChangeCoinReq{
			UserId: in.UserId,
			Change: -titleCfg.PriceDay,
		}, pbVars.UserCoinChangedReason_BuyEliteUnit); err != nil {
			return err
		}

		if err = l.svcCtx.UserTitleModel.Addon(l.ctx, nil, in.UserId, titleCfg.Id, titleCfg.Type, 24*time.Hour, false); err != nil {
			return errors.Wrapf(nerr.NewWithErr(err), "兑换称号失败 err:%v", err)
		}

		return nil
	}); err != nil {
		return nil, err
	}
	return resp, nil
}
